//! # User Web Socket controller
//!
//! Handles the WebSocket offered to the users

use crate::api_data::res_get_ws_token::ResGetWsToken;
use crate::constants::WS_ACCESS_TOKEN_LENGTH;
use crate::data::base_request_handler::BaseRequestHandler;
use crate::data::error::ResultBoxError;
use crate::data::http_request_handler::HttpRequestHandler;
use crate::utils::crypt_utils::rand_str;
use crate::utils::date_utils::time;

/// WebSocket access tokens list
mod ws_tokens_list {
    use std::sync::Arc;
    use std::sync::Mutex;

    use crate::constants::WS_ACCESS_TOKEN_LIFETIME;
    use crate::data::user::UserID;
    use crate::utils::date_utils::time;

    #[derive(Debug)]
    pub struct WsToken {
        pub time: u64,
        pub client_id: u32,
        pub user_id: UserID,
        pub token: String,
        pub incognito: bool,
    }

    lazy_static! {
        static ref WS_TOKENS: Arc<Mutex<Vec<WsToken>>> = {
            Arc::new(Mutex::new(Vec::new()))
        };
    }

    /// Get the list of WebSocket tokens
    fn get_list() -> Arc<Mutex<Vec<WsToken>>> {
        (*WS_TOKENS).clone()
    }

    /// Remove old entries from the list
    pub fn clean_list() {
        let list = get_list();
        let mut list = list.lock().unwrap();

        while let Some(first) = list.first() {
            if first.time < time() - WS_ACCESS_TOKEN_LIFETIME {
                list.remove(0);
            } else {
                break;
            }
        }
    }

    /// Add a new token to the list
    pub fn add_token(t: WsToken) {
        get_list().lock().unwrap().push(t)
    }

    /// Remove a specific access token from the list & return it
    pub fn take_access_token(t: String) -> Option<WsToken> {
        let list = get_list();
        let mut list = list.lock().unwrap();
        for i in 0..list.len() {
            if list[i].token == t {
                return Some(list.remove(i));
            }
        }

        None
    }
}

/// Get a WebSocket access token
pub fn get_token(r: &mut HttpRequestHandler) -> ResultBoxError {
    ws_tokens_list::clean_list();

    let access_token = rand_str(WS_ACCESS_TOKEN_LENGTH);

    let token = ws_tokens_list::WsToken {
        user_id: r.user_id()?,
        client_id: r.api_client().id,
        time: time(),
        token: access_token.to_string(),
        incognito: r.post_bool_opt("incognito", false),
    };

    ws_tokens_list::add_token(token);

    r.set_response(ResGetWsToken::new(access_token))
}